/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.vew.bmvew.bmvew.sender;
 
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Die Klasse zum Versenden von Daten an bestimmte Objekte. 
 * Sie implentiert das ClientSenderInterface welches die Methoden zum Versenden der Daten 
 * bereitstellt.
 * 
 * @version $Revision: 1.1 $ / $Date: 2009/09/24 12:50:46 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class Sender implements ClientSenderInterface
{
  
  /**
   * Die Daten EreignisKalenderAnfrage
   */
  private ClientDavInterface _connection;
  
  /**
   * Die Daten EreignisKalenderAnfrage
   */
  private SenderRole _role;

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfügung
   */
  private Debug _debug;

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfügung
   */
  private Boolean _flag;


  /**
   * Konstruktor der Klasse
   *
   * @param connection DaV
   */
  public Sender(ClientDavInterface connection)
  {
    _connection = connection;
    _debug = Debug.getLogger();
  }



  /**
   * Konstruktor der Klasse
   *
   * @param connection DaV
   * @param role Role
   */
  public Sender(ClientDavInterface connection, SenderRole role)
  {
    _connection = connection;
    _role = role;
    _debug = Debug.getLogger();
  }

  /**
   * Das Ergebis der Anfrage wird ermittelt, aufbereitet und versendet
   * @param so SystemObject
   * @param atg AttributeGroup
   * @param asp Aspect
   * @param data Data
   *
   * @return true, wenn das Versenden der Daten erfolgreich war
   */
  public Boolean sende(SystemObject so, AttributeGroup atg, Aspect asp, Data data)
  {

    DataDescription dd = new DataDescription(atg, asp);

    ResultData resultData = new ResultData(so, dd, _connection.getTime(), data);

    try
    {
      _debug.fine("send1 : " + resultData.getData());
      _connection.sendData(resultData);

    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      try
      {
        _connection.subscribeSender(this, resultData.getObject(), dd, _role);
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      try
      {

        _debug.fine("send2 : " + resultData.getData());
        _connection.sendData(resultData);
      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return true;

  }

  /**
   * Das Ergebis der Anfrage wird ermittelt, aufbereitet und versendet
   * @param so SystemObject
   * @param atg AttributeGroup
   * @param asp Aspect
   * @param data Data
   * @param role Role
   *
   * @return true, wenn das Versenden der Daten erfolgreich war
   */
  public Boolean sende(SystemObject so, AttributeGroup atg, Aspect asp, Data data, SenderRole role)
  {

    DataDescription dd = new DataDescription(atg, asp);

    ResultData resultData = new ResultData(so, dd, _connection.getTime(), data);

    try
    {
      _debug.fine("send1 : " + resultData.getData());
      _connection.sendData(resultData);

    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      try
      {
        _connection.subscribeSender(this, resultData.getObject(), dd, role);
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      try
      {
        _debug.fine("send2 : " + resultData.getData());
        _connection.sendData(resultData);
      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return true;

  }


  /**
   * Das Ergebnis der Anfrage wird ermittelt, aufbereitet und versendet
   * @param resultData Data
   * @param dd DataDescription
   *
   * @return true, wenn das Versenden der Daten erfolgreich war
   */
  public Boolean sende(ResultData resultData, DataDescription dd)
  {

    try
    {
      _debug.fine("send1 : " + resultData.getData());
      _connection.sendData(resultData);
    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      try
      {
        _connection.subscribeSender(this, resultData.getObject(), dd, _role);
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      try
      {
        _debug.fine("send2 : " + resultData.getData());
        _connection.sendData(resultData);

      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return true;

  }


  /**
   * Abstrakte Methode, die implementiert werden muss: Übernimmt das tatsächliche Senden der Daten
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    _debug.fine("Sender : Objekt " + object.getPid() + " dd " + dataDescription + " state " + state);


  }

  /**
   * Abstrakte Methode, die implementiert werden muss: Zeigt, ob die Sendesteuerung erwünscht ist.
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

}
